package no.statnett.ecp.edx;

import no.statnett.ecp.utils.Const;
import no.statnett.ecp.utils.EcpEdxLog;
import no.statnett.ecp.utils.Options;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZonedDateTime;
import java.time.format.DateTimeParseException;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Stream;

import static java.time.ZoneOffset.UTC;


public class EDXMonitor {


  public static final String VERSION = "v1.1.0";
  private static final Flow flowSnd = new Flow(true);
  private static final Flow flowRcv = new Flow(false);

  private static long oldestEcpTms = Long.MAX_VALUE;
  private static long oldestEdxTms = Long.MAX_VALUE;
  private static ZonedDateTime oldestEcpZDT = null;
  private static ZonedDateTime oldestEdxZDT = null;
  private static int interval = 1; // default is 1 minute interval, thus we examine the last measureable minute
  private static boolean debug = false;


  private static void usage() {
    System.out.println("EDXMonitor (EDX) " + VERSION + " can monitor the flow of messages between ECP and EDX.");
    System.out.println("Usage  : java -jar ekit.jar EDX [OPTION] <ecp-log> <edx-log>\n");
    System.out.println("\nThe tool is compatible with ECP 4.14+ - it depends upon certain log events. ");
    System.out.println("The tool will monitor the traffic between ECP and EDX, and will print alerts if");
    System.out.println("there is a discrepancy between the traffic going through ECP and EDX in the last interval");
    System.out.println("examined. The expected way to run the tool is by using a crontab every -i minutes (default");
    System.out.println("evert minute)");
    System.out.println("\nUsage  : java -jar ekit.jar EDX [OPTION] <ecp-log> <edx-log>\n");
    System.out.println(" OPTION:");
    System.out.println("     -i<interval>      : Specifies which interval this tool is run. Default is 1 minute");
    System.out.println("\nExample - run this every minute, will monitor if there is a discrepancy in the last minute:");
    System.out.println("java -jar ekit.jar EDX /var/log/ecp /var/log/edx");
    System.out.println("\nExample - run this every 5 minutes, will monitor if there is a discrepancy in the last 5 minutes:");
    System.out.println("java -jar ekit.jar EDX -i5 /var/log/ecp /var/log/edx");
  }

  private static List<String> parseOptions(String[] initialArgs) {
    List<String> initArgsList = new ArrayList<>(Arrays.asList(initialArgs));
    interval = Options.parseInt(initArgsList, "-i", 1, 60, 1);
    debug = Options.parseBoolean(initArgsList, "-d");
    return initArgsList;
  }

  public static void main(String[] args) {
    try {

      // Initialize some key variables
      ZonedDateTime nowZDT = Instant.now().atZone(UTC);
      oldestEcpZDT = nowZDT;
      oldestEdxZDT = nowZDT;

      // Read the command line arguments
      List<String> mandatoryArgs = parseOptions(args);
      if (mandatoryArgs.size() != 2) {
        usage();
        System.err.println("\n\nExpects two files to be specified (found " + mandatoryArgs.size() + " files): the ECP-log file and the EDX-log file");
        System.exit(1);
      }
      File ecpLog = Options.checkFile(mandatoryArgs.get(0), false);
      File edxLog = Options.checkFile(mandatoryArgs.get(1), false);

      // Process the log files
      processPlainText(ecpLog, null, null, true);
      processPlainText(edxLog, null, null, false);

      // The logs need to have data that cover from requireZDT and forward
      ZonedDateTime requireZDT = nowZDT.minusMinutes(interval + 3); // default is 1 minute interval -> need 4 minutes of logs, to make sure we have 3 full minutes, to make sure we have 1 minute "on each side" of the measurement window
      if (oldestEcpZDT.isAfter(requireZDT) || oldestEdxZDT.isAfter(requireZDT)) {
        // This happens when logs rotate, we will not have enough data - but's it's not a big deal (if there is a problem in the flow between ECP and EDX, it will be detected later)
        System.out.println(makeEmptyLogLine(nowZDT, "INFO ", interval));
        System.exit(0);
      }

      // We have logs that cover the entire measurment periode (usually 3 minutes) - and can perform the measurement
      flowSnd.calculate(nowZDT, interval);
      flowRcv.calculate(nowZDT, interval);

      // Retrieve the data from the measurements
      int diffSnd = flowSnd.diff();
      Measurement fmSnd = flowSnd.getMain();
      int diffRcv = flowRcv.diff();
      Measurement fmRcv = flowRcv.getMain();

      // Print the logline
      String logLevel = (diffSnd >= 0 && diffRcv >= 0) ? "INFO " : (diffSnd < -1 || diffRcv < -1) ? "ERROR" : "WARN ";
      ZonedDateTime measurementZDT = nowZDT.minusMinutes(2).truncatedTo(ChronoUnit.MINUTES);
      System.out.println(makeLogline(measurementZDT, logLevel, interval, fmSnd, fmRcv));

      if (debug) {
        flowSnd.debug();
        System.out.println("/n-----------/n");
        flowRcv.debug();
      }

    } catch (Throwable t) {
      System.err.println("Error occurred: " + t);
    }
  }

  private static long processPlainText(File log, Long startProcessTms, Long endProcessTms, boolean ecp) {
    try (Stream<String> stream = Files.lines(log.toPath(), StandardCharsets.UTF_8)) {
      stream.forEach(s -> {
        processLogLine(startProcessTms, endProcessTms, s, ecp);
      });
    } catch (IOException e) {
      e.printStackTrace();
    }
    return log.length();
  }

  private static void processLogLine(Long startProcessTms, Long endProcessTms, String s, boolean ecp) {
    try {
      if (s != null && s.length() > 20 && (s.startsWith("202"))) {
        long yyyyMMddHHmmSS = Long.parseLong(s.substring(0, 19).replaceAll("[^\\d]", ""));
        long yyyyMMddHHmm = Long.parseLong(s.substring(0, 16).replaceAll("[^\\d]", ""));
        if (startProcessTms != null && yyyyMMddHHmmSS < startProcessTms)
          return;
        if (endProcessTms == null || yyyyMMddHHmmSS < endProcessTms) {
          if (ecp) {
            if (s.contains("ID=MSG-08") && s.contains("STANDARD_MESSAGE")) { // Receive from BROKER into ECP, on its way to EDX
              flowRcv.addEcpId(yyyyMMddHHmm, EcpEdxLog.findMessageIDInLogLine(s));
            } else if (s.contains("ID=MSG-11") && s.contains("STANDARD_MESSAGE")) { // Send from ECP to BROKER (upload to broker)
              flowSnd.addEcpId(yyyyMMddHHmm, EcpEdxLog.findMessageIDInLogLine(s));
            }
          } else {
            if (s.contains("routed to endpoint") && !s.contains("toolbox-gateway")) { // EDX -> BA (RECEIVE)
              flowRcv.addEdxId(yyyyMMddHHmm, EcpEdxLog.findEdxMessageIDInLogLine(s));
            } else if (s.contains("ID=MSG-01")) { // EDX -> ECP (4.14+)
              flowSnd.addEdxId(yyyyMMddHHmm, EcpEdxLog.findEdxMessageIDInLogLine(s));
            }
          }
        }
        if (ecp) {
          if (yyyyMMddHHmmSS < oldestEcpTms) {
            oldestEcpZDT = LocalDateTime.parse("" + yyyyMMddHHmmSS, Const.tmsNumberSec).atZone(UTC);
            oldestEcpTms = yyyyMMddHHmmSS;
          }
        } else {
          if (yyyyMMddHHmmSS < oldestEdxTms) {
            oldestEdxZDT = LocalDateTime.parse("" + yyyyMMddHHmmSS, Const.tmsNumberSec).atZone(UTC);
            oldestEdxTms = yyyyMMddHHmmSS;
          }
        }
      }
    } catch (NumberFormatException |
             DateTimeParseException ex) {
      System.err.println("Error occurred: " + ex + " with input text: " + s);
    }
  }

  public static String makeEmptyLogLine(ZonedDateTime tmsMsg, String loglevel, int interval) {
    String sb =
        tmsMsg.format(Const.utcTmsMillisec) + " " + loglevel + " " +
            "minutes=" + String.format("%-3s", interval + ",") + " " + // room for up to 2 digit minutes
            "ecpSnd=" + String.format("%-6s", "N/A,") + " " +
            "edxSnd=" + String.format("%-6s", "N/A,") + " " +
            "diffSnd=" + String.format("%-6s", "0,") + " " +
            "ecpRcv=" + String.format("%-6s", "N/A,") + " " +
            "edxRcv=" + String.format("%-6s", "N/A,") + " " +
            "diffRcv=" + String.format("%-6s", "0");
    return sb;
  }


  public static String makeLogline(ZonedDateTime tmsMsg, String loglevel, int interval, Measurement fmSnd, Measurement fmRcv) {
    String sb =
        tmsMsg.format(Const.utcTmsMillisec) + " " + loglevel + " " +
            "minutes=" + String.format("%-3s", interval + ",") + " " + // room for up to 2 digit minutes
            "ecpSnd=" + String.format("%-6s", fmSnd.getEcpTraffic() + ",") + " " + // room for up to 5 digit traffic
            "edxSnd=" + String.format("%-6s", fmSnd.getEdxTraffic() + ",") + " " + // room for up to 5 digit traffic
            "diffSnd=" + String.format("%-6s", fmSnd.diff() + ",") + " " + // room for up to 5 digit diff
            "ecpRcv=" + String.format("%-6s", fmRcv.getEcpTraffic() + ",") + " " + // room for up to 5 digit traffic
            "edxRcv=" + String.format("%-6s", fmRcv.getEdxTraffic() + ",") + " " + // room for up to 5 digit traffic
            "diffRcv=" + String.format("%-6s", fmRcv.diff());
    return sb;
  }
}

