package no.statnett.ecp.edx;

import no.statnett.ecp.utils.EcpHTTP;

import java.io.IOException;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;


/**
 * This class was an attempt to extract information about messages that has been delayed, as seen/monitored from EDX (through the /api/rest/messages/delayed API).
 * However, it became clear that this was not the best way to deal with such issues, because of the following:
 *
 * 1) Even if we detect delayed messages sent from EDX, we would not know if these delays were caused by problems in EDX, or in our/local ECP or the remote ECP. Since
 * the main purpose of this monitoring is for alarms, it would cause many false alarms, actually almost all alarms would be caused by problems outside EDX.
 * 2) The API offered did not give us the information we needed, because it would give us information about the last *initiated* (attempt to send) messages within
 * a given time range. If we monitored the past 60 minutes (every 5 minute), we would not know if "delay-situation" had been resolved after 61 minutes, unless new messages
 * were sent. Thus we would not have an easy time knowing when a situation was resolved.
 * 3) If we tried to monitor longer timespans (to circumvent problem in 2)) we would put a heavy load on the EDX server - the monitoring would scan the entire table
 * 4) The API would return max 1000 items at the time, we would have to page through to get all results - unnecessary complicated
 *
 * Finally, there are other ways to monitor this issue, like using the EPA tool, the QM tool, the EML tool (depending on what you want). The EPA tool underwent considerable
 * change and forked off into the EDXFlow tool in order to offer alarms if there is a diff detected in the traffic passing through EDX and ECP.
 * Combined with the QM tool this should provide us with enough information to always know whether there is a problem in EDX or not.
 *
 * This class is kept for historical reasons, and to show that we have considered this approach.
 */
public class EDXMonitorRejected {
  public static final String VERSION = "0.1.0";

  private static String user;
  private static String pw;
  private static URL url;


  public static void main(String[] args) throws IOException, NoSuchAlgorithmException, KeyManagementException {
    URL initialURL = new URL(args[0]);
    // Delayed URL                     : /api/rest/messages/delayed/page/0?count=10&filter=all&getMessages=true&orderBy=time&orderDir=asc&range=200 (filter can be red, orange, all) (range is the number of minutes going back from now)
    // Messages URL - based on N-last  : /api/rest/messages/waiting/page/0?count=100&orderBy=time&orderDir=desc&senderReceiverSwitch=AND
    // Messages URL - based on timespan: /api/rest/messages/waiting/page/total?dateFrom=2024-12-22T23:00:00.000Z&dateTo=2024-12-24T22:59:59.999Z&senderReceiverSwitch=AND
    url = new URL(initialURL.getProtocol(), initialURL.getHost(), initialURL.getPort(), "/api/rest/messages/delayed/page/0?count=10&filter=all&getMessages=true&orderBy=time&orderDir=asc&range=200");
    if (initialURL.getUserInfo() != null) {
      user = initialURL.getUserInfo().split(":")[0];
      pw = initialURL.getUserInfo().split(":")[1];
    }
    String response = EcpHTTP.getECPHTTResponse("GET", url, user, pw, null);
    // the response is something like this, max 1000 items covering the last 24 hours, probably the 100c0 latest messages:
    // {"items":[
    //   {"id":339320,
    //   "status":"Received",
    //   "time":"2024-11-01T14:41:03.35787923",
    //   "direction":"Send",
    //   "sender":"50V-SN-TEST--ATK",
    //   "senderName":"Statnett",
    //   "receiver":"50V-SN-TEST2-AT7",
    //   "receiverName":"Statnett",
    //   "messageType":"TEST",
    //   "edxKind":"PLAIN",
    //   "baMessageId":null,
    //   "baCorrelationId":null,
    //   "senderApplication":"MORTEN",
    //   "signal":"green"}
    // ],"itemsCount":1,"pageNum":0,"pageSize":10,"allCount":1,"orangeCount":0,"redCount":0}
    // There could be many items in the items array

    // We're interested in the following:
    // * Only status !=m Received
    // * Only direction = Send
    // * time
    // * receiver
    // * receiverName

    // From these we can make a list like this:
    // receiver/receiverName: last time with delay, number of delays last 24h
    // total: last time with delay, number of delays last 24h

    // Find the oldest "last time" and ask for all messages since that time (filter is not set at all). For each receiver that had delays, find newest
    // message. Compare if this newest message is newer than the "last time" (if so, the status of that message is OK)

    // A little later: ... no this is not the way forward (see comment at the top of the class)

    System.out.println(response);
  }
}
