package no.statnett.ecp.edx;

import no.statnett.ecp.utils.Const;

import java.time.ZonedDateTime;
import java.util.*;

public class Flow {
  // This class will store the traffic (PLAIN MSG/STANDARD MESSAGES) passing through ECP and EDX in ONE direction, divided in
  // "measurements". Each measurement is populated by reading the edx and ecp log, divided by minute.

  // Once all measurements are populated, the calculate-method will be called to make the comparison of most recent measurement.
  // The main idea is that the traffic going through both ECP and EDX should be the same, otherwise there is a problem. There could of course
  // be a minute-shift between the two, but that should be the only difference - and we must be able to detect that (and ignore it).

  private boolean snd;
  private Map<Long, Measurement> allMeasurements = new HashMap<>();
  private Measurement main;
  private Measurement before;
  private Measurement after;
  private int diff = 0;

  public Flow(boolean snd) {
    this.snd = snd;
  }

  public void addEcpId(long yyyyMMddHHmm, String ecpId) {
    Measurement measurement = allMeasurements.get(yyyyMMddHHmm);
    if (measurement == null) {
      measurement = new Measurement(yyyyMMddHHmm, snd);
      allMeasurements.put(yyyyMMddHHmm, measurement);
    }
    measurement.addEcpId(ecpId);
  }

  public void addEdxId(long yyyyMMddHHmm, String edxId) {
    Measurement measurement = allMeasurements.get(yyyyMMddHHmm);
    if (measurement == null) {
      measurement = new Measurement(yyyyMMddHHmm, snd);
      allMeasurements.put(yyyyMMddHHmm, measurement);
    }
    measurement.addEdxId(edxId);
  }

  private Measurement getMeasurement(long yyyyMMddHHmm) {
    return allMeasurements.get(yyyyMMddHHmm) == null ? new Measurement(yyyyMMddHHmm, snd) : allMeasurements.get(yyyyMMddHHmm);
  }

  public void calculate(ZonedDateTime nowZDT, int interval) {
    // Find the minute *after* the minute we want to measure - that should always be the previous minute (we expect that to be a complete minute)
    long yyyyMMddHHmmAfter = Long.parseLong(nowZDT.minusMinutes(1).format(Const.tmsNumberMinute));
    after = getMeasurement(yyyyMMddHHmmAfter);

    // The actual measurement window (usually 1 minute long, but it could be longer if interval > 1)
    long yyyyMMddHHmmMeasure = Long.parseLong(nowZDT.minusMinutes(2).format(Const.tmsNumberMinute));
    main = getMeasurement(yyyyMMddHHmmMeasure);
    for (int i = 1; i < interval; i++) {
      yyyyMMddHHmmMeasure = Long.parseLong(nowZDT.minusMinutes(2 + i).format(Const.tmsNumberMinute));
      main.add(getMeasurement(yyyyMMddHHmmMeasure));
    }

    // The minute *before* the measurement window
    long yyyyMMddHHmmBefore = Long.parseLong(nowZDT.minusMinutes(2 + interval).format(Const.tmsNumberMinute));
    before = getMeasurement(yyyyMMddHHmmBefore);

    // We calculate the diff, and try to find combinations of the three measurements that will give us a diff of 0 or greater
    // If we can't find such a combination, we return the diff of the main measurement
    if (main.diff() > 0)
      diff = main.diff();
    else if (main.diff() + after.diff() >= 0)
      diff = main.diff() + after.diff();
    else if (main.diff() + before.diff() >= 0)
      diff = main.diff() + before.diff();
    else if (main.diff() + before.diff() + after.diff() >= 0)
      diff = main.diff() + before.diff() + after.diff();
    else
      diff = main.diff();
  }

  public int diff() {
    return diff;
  }

  public Measurement getMain() {
    return main;
  }

  public void debug() {
    // Print all measurements sorted by key in numerical ascending order
    for (Long tms : allMeasurements.keySet().stream().sorted().toArray(Long[]::new)) {
      System.out.println("Tms: " + tms + " -> " + allMeasurements.get(tms).getEcpTraffic() + " / " + allMeasurements.get(tms).getEdxTraffic());
    }
  }


}
